import { scaleLinear, axisBottom, axisLeft, extent } from 'd3';

export const scatterPlot = (selection, props) => {
	const {
  	xValue,
  	xAxisLabel,
  	yValue,
  	yAxisLabel,
  	circleRadius,
  	margin,
  	width,
  	height,
  	data } = props;
  
  /* The inner width and height of the chart. */
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  const xScale = scaleLinear()
  	.domain(extent(data, xValue))
  	.range([0, innerWidth])
  	.nice();
  
  /* The linear y Scale. Note that the smallest value of 
  the domain maps to the largest value of range, i.e. innerHeight,
  on the screen, and hence corresponds to the bottom line. */
  const yScale = scaleLinear()
  	.domain(extent(data, yValue))
  	.range([innerHeight, 0])
  	.nice();
  
  /* update selection of group element g */
  const g = selection.selectAll('.container').data([null]);
  /* enter selection of group element g with class container*/
  const gEnter = g.enter().append('g')
  								.attr('class', 'container');
  /* Translate g according to the margin set in the props. */
  gEnter
  	.merge(g)
      .attr('transform', `translate(${margin.left}, ${margin.top})`);
  
  /* Set the x-axis and y-axis, their corresponding 
  tick size (-innerHeight & width) so that they become the 
  gridliines, as well as the padding. */
  const xAxis = axisBottom(xScale)
  	.tickSize(-innerHeight)
  	.tickPadding(10);
    
  const yAxis = axisLeft(yScale)
  	.tickSize(-innerWidth)
  	.tickPadding(5);
  
  /* The y-axis group element, with class y-axis. */
  const yAxisG = g.select('.y-axis');
  const yAxisGEnter = gEnter
    .append('g')
  		.attr('class', 'y-axis');
  
  yAxisG
  	.merge(yAxisGEnter)
    	.call(yAxis)
      /* Removes the border of the chart. */
			.selectAll('.domain').remove();
  
  const yAxisLabelText = yAxisGEnter.append('text')
      .attr('transform', `rotate(-90)`)
      .attr('class', 'axis-label')
      .attr('y', -35)
      .attr('fill', 'black')
      .style('text-anchor', 'middle')
  	/* yAxislabel needs to be updated every time new data is loaded */
  	.merge(yAxisG.select('.axis-label'))
      .attr('x', -innerHeight/2)
      .text(yAxisLabel);
  
  /* The x-axis group element, with class x-axis. */
  const xAxisG = g.select('.x-axis');
  const xAxisGEnter = gEnter
    .append('g')
  		.attr('class', 'x-axis');
  xAxisG
  	.merge(xAxisGEnter)
      /* Need this transform because the axis is initially set at the top of the chart. */
  		.attr('transform', `translate(0, ${innerHeight})`)
    	.call(xAxis)
			.selectAll('.domain').remove();
  
  const xAxisLabelText = xAxisGEnter.append('text')
      .attr('class', 'axis-label')
      .attr('y', 35)
      .attr('fill', 'black')
    /* xAxislabel needs to be updated every time new data is loaded */
  	.merge(xAxisG.select('.axis-label'))
      .attr('x', innerWidth/2)
      .text(xAxisLabel);
  
  /* "circles" is part of the enter AND update selection
  of the g element, since we need to draw these circles every
  single time.
  */
	const circles = g.merge(gEnter).selectAll('circle').data(data);
  circles.enter().append('circle')
    .merge(circles)
  		/* ENTER & UPDATE @ BEFORE TRANSITION */
      .attr('cy', innerHeight/2)
      .attr('cx', innerWidth/2)
  		.attr('r', 0)
  	.transition().duration(1000)
  	/* delay 2 milliseconds for each circle */
  	.delay((d, i) => i * 2)
      .attr('cy', d => yScale(yValue(d)))
      .attr('cx', d => xScale(xValue(d)))
      .attr('r', circleRadius);

}